==> symbol-processing-aa-embeddable-2.1.0-1.0.29-sources.jar: org/jetbrains/kotlin/light/classes/symbol/classes/SymbolLightClassForValueClass.kt <==
/*
 * Copyright 2010-2024 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.light.classes.symbol.classes

import ksp.com.intellij.psi.PsiField
import ksp.com.intellij.psi.PsiManager
import ksp.com.intellij.psi.PsiMethod
import ksp.org.jetbrains.kotlin.analysis.api.KaSession
import ksp.org.jetbrains.kotlin.analysis.api.projectStructure.KaModule
import ksp.org.jetbrains.kotlin.analysis.api.symbols.KaKotlinPropertySymbol
import ksp.org.jetbrains.kotlin.analysis.api.symbols.KaNamedClassSymbol
import ksp.org.jetbrains.kotlin.analysis.api.symbols.KaNamedFunctionSymbol
import ksp.org.jetbrains.kotlin.analysis.api.symbols.KaPropertySymbol
import ksp.org.jetbrains.kotlin.analysis.api.symbols.KaSymbolVisibility
import ksp.org.jetbrains.kotlin.analysis.api.symbols.pointers.KaSymbolPointer
import ksp.org.jetbrains.kotlin.analysis.api.symbols.pointers.symbolPointerOfType
import ksp.org.jetbrains.kotlin.lexer.KtTokens
import ksp.org.jetbrains.kotlin.light.classes.symbol.cachedValue
import ksp.org.jetbrains.kotlin.light.classes.symbol.fields.SymbolLightField
import ksp.org.jetbrains.kotlin.psi.KtClassOrObject

internal class SymbolLightClassForValueClass : SymbolLightClassForClassOrObject {
    constructor(
        classOrObject: KtClassOrObject,
        ktModule: KaModule,
    ) : this(
        classOrObjectDeclaration = classOrObject,
        classSymbolPointer = classOrObject.symbolPointerOfType(),
        ktModule = ktModule,
        manager = classOrObject.manager,
    ) {
        require(classOrObject.hasModifier(KtTokens.INLINE_KEYWORD) || classOrObject.hasModifier(KtTokens.VALUE_KEYWORD))
    }

    constructor(
        ktAnalysisSession: KaSession,
        ktModule: KaModule,
        classSymbol: KaNamedClassSymbol,
        manager: PsiManager,
    ) : super(
        ktAnalysisSession = ktAnalysisSession,
        ktModule = ktModule,
        classSymbol = classSymbol,
        manager = manager,
    ) {
        require(classSymbol.isInline)
    }

    private constructor(
        classOrObjectDeclaration: KtClassOrObject?,
        classSymbolPointer: KaSymbolPointer<KaNamedClassSymbol>,
        ktModule: KaModule,
        manager: PsiManager,
    ) : super(
        classOrObjectDeclaration = classOrObjectDeclaration,
        classSymbolPointer = classSymbolPointer,
        ktModule = ktModule,
        manager = manager,
    )

    override val ownConstructors: Array<PsiMethod> get() = PsiMethod.EMPTY_ARRAY

    override fun getOwnMethods(): List<PsiMethod> = cachedValue {
        withClassSymbol { classSymbol ->
            val result = mutableListOf<PsiMethod>()

            // Value classes have overridden methods
            val applicableDeclarations = classSymbol.declaredMemberScope
                .callables
                .filter {
                    (it as? KaPropertySymbol)?.isOverride == true || (it as? KaNamedFunctionSymbol)?.isOverride == true
                }
                .filterNot {
                    it.hasTypeForValueClassInSignature()
                }

            createMethods(applicableDeclarations, result, suppressStatic = false)
            generateMethodsFromAny(classSymbol, result)

            val propertySymbol = propertySymbol(classSymbol)

            // Only public properties have accessors for value classes
            if (propertySymbol != null && propertySymbol.visibility == KaSymbolVisibility.PUBLIC) {
                // (inline or) value class primary constructor must have only final read-only (val) property parameter
                // Even though the property parameter is mutable (for some reasons, e.g., testing or not checked yet),
                // we can enforce immutability here.
                createPropertyAccessors(result, propertySymbol, isTopLevel = false, isMutable = false)
            }

            addDelegatesToInterfaceMethods(result, classSymbol)

            result
        }
    }

    override fun getOwnFields(): List<PsiField> = cachedValue {
        withClassSymbol { classSymbol ->
            buildList {
                val nameGenerator = SymbolLightField.FieldNameGenerator()
                addCompanionObjectFieldIfNeeded(this, classSymbol)

                addFieldsFromCompanionIfNeeded(this, classSymbol, nameGenerator)

                propertySymbol(classSymbol)?.let {
                    createAndAddField(it, nameGenerator, isStatic = false, result = this)
                }
            }
        }
    }

    override fun copy(): SymbolLightClassForValueClass =
        SymbolLightClassForValueClass(classOrObjectDeclaration, classSymbolPointer, ktModule, manager)
}

private fun KaSession.propertySymbol(classSymbol: KaNamedClassSymbol): KaKotlinPropertySymbol? {
    return classSymbol.declaredMemberScope
        .constructors
        .singleOrNull { it.isPrimary }
        ?.valueParameters
        ?.singleOrNull()
        ?.generatedPrimaryConstructorProperty
}

==> symbol-processing-aa-embeddable-2.2.0-2.0.2-sources.jar: org/jetbrains/kotlin/light/classes/symbol/classes/SymbolLightClassForValueClass.kt <==
/*
 * Copyright 2010-2025 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.light.classes.symbol.classes

import ksp.com.intellij.psi.PsiField
import ksp.com.intellij.psi.PsiManager
import ksp.com.intellij.psi.PsiMethod
import ksp.org.jetbrains.kotlin.analysis.api.KaImplementationDetail
import ksp.org.jetbrains.kotlin.analysis.api.KaSession
import ksp.org.jetbrains.kotlin.analysis.api.projectStructure.KaModule
import ksp.org.jetbrains.kotlin.analysis.api.symbols.*
import ksp.org.jetbrains.kotlin.analysis.api.symbols.pointers.KaSymbolPointer
import ksp.org.jetbrains.kotlin.lexer.KtTokens
import ksp.org.jetbrains.kotlin.light.classes.symbol.cachedValue
import ksp.org.jetbrains.kotlin.light.classes.symbol.fields.SymbolLightField
import ksp.org.jetbrains.kotlin.light.classes.symbol.methods.SymbolLightAccessorMethod.Companion.createPropertyAccessors
import ksp.org.jetbrains.kotlin.psi.KtClassOrObject

internal class SymbolLightClassForValueClass : SymbolLightClassForClassOrObject {
    @OptIn(KaImplementationDetail::class)
    constructor(
        classOrObject: KtClassOrObject,
        ktModule: KaModule,
    ) : this(
        classOrObjectDeclaration = classOrObject,
        classSymbolPointer = classOrObject.createSymbolPointer(ktModule),
        ktModule = ktModule,
        manager = classOrObject.manager,
    ) {
        require(classOrObject.hasModifier(KtTokens.INLINE_KEYWORD) || classOrObject.hasModifier(KtTokens.VALUE_KEYWORD))
    }

    constructor(
        ktAnalysisSession: KaSession,
        ktModule: KaModule,
        classSymbol: KaNamedClassSymbol,
        manager: PsiManager,
    ) : super(
        ktAnalysisSession = ktAnalysisSession,
        ktModule = ktModule,
        classSymbol = classSymbol,
        manager = manager,
    ) {
        require(classSymbol.isInline)
    }

    private constructor(
        classOrObjectDeclaration: KtClassOrObject?,
        classSymbolPointer: KaSymbolPointer<KaNamedClassSymbol>,
        ktModule: KaModule,
        manager: PsiManager,
    ) : super(
        classOrObjectDeclaration = classOrObjectDeclaration,
        classSymbolPointer = classSymbolPointer,
        ktModule = ktModule,
        manager = manager,
    )

    override val ownConstructors: Array<PsiMethod> get() = PsiMethod.EMPTY_ARRAY

    override fun getOwnMethods(): List<PsiMethod> = cachedValue {
        withClassSymbol { classSymbol ->
            val result = mutableListOf<PsiMethod>()

            // Value classes have overridden methods
            val applicableDeclarations = classSymbol.declaredMemberScope
                .callables
                .filter {
                    (it as? KaPropertySymbol)?.isOverride == true || (it as? KaNamedFunctionSymbol)?.isOverride == true
                }

            createMethods(this@SymbolLightClassForValueClass, applicableDeclarations, result)
            generateMethodsFromAny(classSymbol, result)

            val propertySymbol = propertySymbol(classSymbol)

            // Only public properties have accessors for value classes
            if (propertySymbol != null && propertySymbol.visibility == KaSymbolVisibility.PUBLIC) {
                // (inline or) value class primary constructor must have only final read-only (val) property parameter
                // Even though the property parameter is mutable (for some reasons, e.g., testing or not checked yet),
                // we can enforce immutability here.
                createPropertyAccessors(this@SymbolLightClassForValueClass, result, propertySymbol, isTopLevel = false, isMutable = false)
            }

            addDelegatesToInterfaceMethods(result, classSymbol)

            result
        }
    }

    override fun getOwnFields(): List<PsiField> = cachedValue {
        withClassSymbol { classSymbol ->
            buildList {
                val nameGenerator = SymbolLightField.FieldNameGenerator()
                addCompanionObjectFieldIfNeeded(this, classSymbol)

                addFieldsFromCompanionIfNeeded(this, classSymbol, nameGenerator)

                propertySymbol(classSymbol)?.let {
                    createAndAddField(this@SymbolLightClassForValueClass, it, nameGenerator, isStatic = false, result = this)
                }
            }
        }
    }

    override fun copy(): SymbolLightClassForValueClass =
        SymbolLightClassForValueClass(classOrObjectDeclaration, classSymbolPointer, ktModule, manager)
}

private fun KaSession.propertySymbol(classSymbol: KaNamedClassSymbol): KaKotlinPropertySymbol? {
    return classSymbol.declaredMemberScope
        .constructors
        .singleOrNull { it.isPrimary }
        ?.valueParameters
        ?.singleOrNull()
        ?.generatedPrimaryConstructorProperty
}
